#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "kingate/fastcgi.h"
#include "kingate/exception.h"

namespace kingate {

	bool fcgi_socket::_initialized = false;

	fcgi_socket::fcgi_socket(const char *s,int bl) 
	    : sock(-1) {
		if(!_initialized) {
		    if( FCGX_Init() )
			throw exception(CODEPOINT,"failed to FCGX_Init()");
		    _initialized = true;
		}
		sock = FCGX_OpenSocket(s,bl);
		if(sock<0)
		    throw exception(CODEPOINT,"failed to FCGX_OpenSocket(");
		// TODO: check if there is a ':', not if it starts with ':'
		if(*s != ':')
		    if(chmod(s,0777)) // XXX: configurable.
			throw exception(CODEPOINT,"failed to chmod()");
	    }
	fcgi_socket::fcgi_socket(int s)
	    : sock(0) {
		if(!_initialized) {
		    if( FCGX_Init() )
			throw exception(CODEPOINT,"failed to FCGX_Init()");
		    _initialized = true;
		}
	    }
	fcgi_socket::~fcgi_socket() {
	    if(sock>=0)
		close(sock);
	}

	fcgi_interface::fcgi_interface(fcgi_socket& s,int f)
	    : sin(&sbin), sout(&sbout), serr(&sberr) {
		if( FCGX_InitRequest(&request,s.sock,f) )
		    throw exception(CODEPOINT,"failed to FCGX_InitRequest()");
		if( FCGX_Accept_r(&request) )
		    throw exception(CODEPOINT,"failed to FCGX_Accept_r()");
		sbin.attach(request.in);
		sbout.attach(request.out);
		sberr.attach(request.err);
		for(char **p = request.envp; *p; p++) {
		    const char *e = strchr(*p,'=');
		    if(!e){
			// XXX: check if we have it already?
			metavars[*p] = string(0);
		    }else{
			int l = e-*p; e++;
			// XXX: check if we have it already?
			metavars[string(*p,l)]=e;
		    }
		}
	    }
	fcgi_interface::~fcgi_interface() {
	    sout.flush();
	    serr.flush();
	    FCGX_Finish_r(&request);
	}

}
